@@ -13,10 +13,10 @@
 ///
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Last changed  : $Date: 2006-05-23 13:14:24 -0600 (Tue, 23 May 2006) $
-// File revision : $Revision: 9998 $
+// Last changed  : $Date$
+// File revision : $Revision$
 //
-// $Id: TDStretch.cpp 9998 2006-05-23 19:14:24Z danielk $
+// $Id$
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -803,36 +803,22 @@ TDStretch * TDStretch::newInstance()
     uExtensions = detectCPUextensions();
 
     // Check if MMX/SSE/3DNow! instruction set extensions supported by CPU
-
-#ifdef ALLOW_MMX
-    // MMX routines available only with integer sample types
-    if (uExtensions & MM_MMX)
-    {
-        return ::new TDStretchMMX;
-    }
-    else
-#endif // ALLOW_MMX
-
-
-#ifdef ALLOW_SSE
-    if (uExtensions & MM_SSE)
+#ifdef ALLOW_SSE3
+    if (uExtensions & MM_SSE3)
     {
         // SSE support
-        return ::new TDStretchSSE;
+        return ::new TDStretchSSE3;
     }
     else
-#endif // ALLOW_SSE
-
-
-#ifdef ALLOW_3DNOW
-    if (uExtensions & MM_3DNOW)
+#endif // ALLOW_SSE3
+#ifdef ALLOW_SSE2
+    if (uExtensions & MM_SSE2)
     {
-        // 3DNow! support
-        return ::new TDStretch3DNow;
+        // SSE support
+        return ::new TDStretchSSE2;
     }
     else
-#endif // ALLOW_3DNOW
-
+#endif // ALLOW_SSE2
     {
         // ISA optimizations not supported, use plain C version
         return ::new TDStretch;
@@ -1023,7 +1009,6 @@ long TDStretch::calcCrossCorrMulti(const short *mixingPos, const short *compare)
 
 #ifdef FLOAT_SAMPLES
 
-
 // Slopes the amplitude of the 'midBuffer' samples so that cross correlation
 // is faster to calculate
 void TDStretch::precalcCorrReferenceStereo()
@@ -1040,6 +1025,28 @@ void TDStretch::precalcCorrReferenceStereo()
     }
 }
 
+#ifdef MULTICHANNEL
+// Slopes the amplitude of the 'midBuffer' samples so that cross correlation
+// is faster to calculate
+void TDStretch::precalcCorrReference()
+{
+    int i,j;
+    float temp, temp2;
+    float *src = pMidBuffer;
+    float *dest = pRefMidBuffer;
+
+    for (i = 0 ; i < (int)overlapLength ; i++) 
+    {
+        temp = (float)i * (float)(overlapLength - i);
+
+        for(j = 0; j < channels; j++)
+        {
+            temp2 = (float)(*src++ * temp);
+            *dest++ = temp2;
+        }
+    }
+}
+#endif
 
 // Slopes the amplitude of the 'midBuffer' samples so that cross correlation
 // is faster to calculate
@@ -1055,7 +1062,6 @@ void TDStretch::precalcCorrReferenceMono()
     }
 }
 
-
 // SSE-optimized version of the function overlapStereo
 void TDStretch::overlapStereo(float *output, const float *input) const
 {
@@ -1077,6 +1083,27 @@ void TDStretch::overlapStereo(float *output, const float *input) const
     }
 }
 
+#ifdef MULTICHANNEL
+// Overlaps samples in 'midBuffer' with the samples in 'input'. 
+void TDStretch::overlapMulti(float *output, const float *input) const
+{
+    int i,j;
+    float temp;
+    const float *ip = input;
+    float *op = output;
+    const float *md = pMidBuffer;
+    float fScale = 1.0f / (float)overlapLength;
+    float fi;
+
+    for (i = 0; i < (int)overlapLength ; i ++) 
+    {
+        temp = (float)(overlapLength - i) * fScale;
+        fi = (float)i * fScale;
+        for(j = 0; j < channels; j++)
+            *op++ = (*ip++ * fi + *md++ * temp);
+    }
+}
+#endif
 
 /// Calculates overlap period length in samples.
 void TDStretch::calculateOverlapLength(uint overlapMs)
@@ -1124,4 +1151,20 @@ double TDStretch::calcCrossCorrStereo(const float *mixingPos, const float *compa
     return corr;
 }
 
+#ifdef MULTICHANNEL
+double TDStretch::calcCrossCorrMulti(const float *mixingPos, const float *compare) const
+{
+    double corr;
+    uint i;
+
+    corr = 0;
+    for (i = channels; i < channels * overlapLength; i++) 
+    {
+        corr += mixingPos[i] * compare[i];
+    }
+
+    return corr;
+}
+#endif
+
 #endif // FLOAT_SAMPLES
